import { CfnEgressOnlyInternetGateway, CfnInternetGateway, CfnNatGateway, CfnRoute, CfnRouteTable, CfnVPNGateway, IRouteTable, ISubnet, IVpcEndpoint, RouterType, VpnConnectionType } from 'aws-cdk-lib/aws-ec2';
import { Construct, IDependable } from 'constructs';
import { Duration, Resource } from 'aws-cdk-lib/core';
import { IVpcV2 } from './vpc-v2-base';
/**
 * Indicates whether the NAT gateway supports public or private connectivity.
 * The default is public connectivity.
 * See: https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-natgateway.html#cfn-ec2-natgateway-connectivitytype
 */
export declare enum NatConnectivityType {
    /**
     * Sets Connectivity type to PUBLIC
     */
    PUBLIC = "public",
    /**
     * Sets Connectivity type to PRIVATE
     */
    PRIVATE = "private"
}
/**
 * Interface to define a routing target, such as an
 * egress-only internet gateway or VPC endpoint.
 */
export interface IRouteTarget {
    /**
     * The type of router used in the route.
     */
    readonly routerType: RouterType;
    /**
     * The ID of the route target.
     */
    readonly routerTargetId: string;
}
/**
 * Properties to define an egress-only internet gateway.
 */
export interface EgressOnlyInternetGatewayProps {
    /**
     * The ID of the VPC for which to create the egress-only internet gateway.
     */
    readonly vpc: IVpcV2;
    /**
     * The resource name of the egress-only internet gateway.
     * @default none
     */
    readonly egressOnlyInternetGatewayName?: string;
}
/**
 * Properties to define an internet gateway.
 */
export interface InternetGatewayProps {
    /**
     * The ID of the VPC for which to create the internet gateway.
     */
    readonly vpc: IVpcV2;
    /**
     * The resource name of the internet gateway.
     * @default none
     */
    readonly internetGatewayName?: string;
}
/**
 * Properties to define a VPN gateway.
 */
export interface VPNGatewayProps {
    /**
     * The type of VPN connection the virtual private gateway supports.
     * @see http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpngateway.html#cfn-ec2-vpngateway-type
     */
    readonly type: VpnConnectionType;
    /**
     * The ID of the VPC for which to create the VPN gateway.
     */
    readonly vpc: IVpcV2;
    /**
     * The private Autonomous System Number (ASN) for the Amazon side of a BGP session.
     * @default none
     */
    readonly amazonSideAsn?: number;
    /**
     * The resource name of the VPN gateway.
     * @default none
     */
    readonly vpnGatewayName?: string;
}
/**
 * Properties to define a NAT gateway.
 */
export interface NatGatewayProps {
    /**
     * The subnet in which the NAT gateway is located.
     */
    readonly subnet: ISubnet;
    /**
     * The ID of the VPC in which the NAT gateway is located.
     * @default none
     */
    readonly vpc?: IVpcV2;
    /**
     * AllocationID of Elastic IP address that's associated with the NAT gateway. This property is required for a public NAT
     * gateway and cannot be specified with a private NAT gateway.
     * @default attr.allocationID of a new Elastic IP created by default
     * //TODO: ADD L2 for elastic ip
     */
    readonly allocationId?: string;
    /**
     * Indicates whether the NAT gateway supports public or private connectivity.
     * @default public
     */
    readonly connectivityType?: NatConnectivityType;
    /**
     * The maximum amount of time to wait before forcibly releasing the
     * IP addresses if connections are still in progress.
     * @default 350 seconds
     */
    readonly maxDrainDuration?: Duration;
    /**
     * The private IPv4 address to assign to the NAT gateway. If you don't provide an
     * address, a private IPv4 address will be automatically assigned.
     * @default none
     */
    readonly privateIpAddress?: string;
    /**
     * Secondary EIP allocation IDs.
     * @default none
     * @see https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html#nat-gateway-creating
     */
    readonly secondaryAllocationIds?: string[];
    /**
     * The number of secondary private IPv4 addresses you
     * want to assign to the NAT gateway.
     *
     * `SecondaryPrivateIpAddressCount` and `SecondaryPrivateIpAddresses` cannot be
     * set at the same time.
     *
     * @default none
     * @see https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html#nat-gateway-creating
     */
    readonly secondaryPrivateIpAddressCount?: number;
    /**
     * Secondary private IPv4 addresses.
     *
     * `SecondaryPrivateIpAddressCount` and `SecondaryPrivateIpAddresses` cannot be
     * set at the same time.
     *
     * @default none
     * @see https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html#nat-gateway-creating
     */
    readonly secondaryPrivateIpAddresses?: string[];
    /**
     * The resource name of the NAT gateway.
     * @default none
     */
    readonly natGatewayName?: string;
}
/**
 * Creates an egress-only internet gateway
 * @resource AWS::EC2::EgressOnlyInternetGateway
 */
export declare class EgressOnlyInternetGateway extends Resource implements IRouteTarget {
    /**
     * The type of router used in the route.
     */
    readonly routerType: RouterType;
    /**
     * The ID of the route target.
     */
    readonly routerTargetId: string;
    /**
     * The egress-only internet gateway CFN resource.
     */
    readonly resource: CfnEgressOnlyInternetGateway;
    constructor(scope: Construct, id: string, props: EgressOnlyInternetGatewayProps);
}
/**
 * Creates an internet gateway
 * @resource AWS::EC2::InternetGateway
 */
export declare class InternetGateway extends Resource implements IRouteTarget {
    /**
     * The type of router used in the route.
     */
    readonly routerType: RouterType;
    /**
     * The ID of the route target.
     */
    readonly routerTargetId: string;
    /**
     * The ID of the VPC for which to create the internet gateway.
     */
    readonly vpcId: string;
    /**
     * The internet gateway CFN resource.
     */
    readonly resource: CfnInternetGateway;
    constructor(scope: Construct, id: string, props: InternetGatewayProps);
}
/**
 * Creates a virtual private gateway
 * @resource AWS::EC2::VPNGateway
 */
export declare class VPNGateway extends Resource implements IRouteTarget {
    /**
     * The type of router used in the route.
     */
    readonly routerType: RouterType;
    /**
     * The ID of the route target.
     */
    readonly routerTargetId: string;
    /**
     * The ID of the VPC for which to create the VPN gateway.
     */
    readonly vpcId: string;
    /**
     * The VPN gateway CFN resource.
     */
    readonly resource: CfnVPNGateway;
    constructor(scope: Construct, id: string, props: VPNGatewayProps);
}
/**
 * Creates a network address translation (NAT) gateway
 * @resource AWS::EC2::NatGateway
 */
export declare class NatGateway extends Resource implements IRouteTarget {
    /**
     * The type of router used in the route.
     */
    readonly routerType: RouterType;
    /**
     * The ID of the route target.
     */
    readonly routerTargetId: string;
    /**
     * Indicates whether the NAT gateway supports public or private connectivity.
     * @default public
     */
    readonly connectivityType?: string;
    /**
     * The maximum amount of time to wait before forcibly releasing the
     * IP addresses if connections are still in progress.
     * @default 350 seconds
     */
    readonly maxDrainDuration?: Duration;
    /**
     * The NAT gateway CFN resource.
     */
    readonly resource: CfnNatGateway;
    constructor(scope: Construct, id: string, props: NatGatewayProps);
}
/**
 * The type of endpoint or gateway being targeted by the route.
 */
export interface RouteTargetProps {
    /**
     * The gateway route target. This is used for targets such as
     * egress-only internet gateway or VPC peering connection.
     * @default none
     */
    readonly gateway?: IRouteTarget;
    /**
     * The endpoint route target. This is used for targets such as
     * VPC endpoints.
     * @default none
     */
    readonly endpoint?: IVpcEndpoint;
}
/**
 * The gateway or endpoint targeted by the route.
 */
export declare class RouteTargetType {
    /**
     * The gateway route target. This is used for targets such as
     * egress-only internet gateway or VPC peering connection.
     * @default none
     */
    readonly gateway?: IRouteTarget;
    /**
     * The endpoint route target. This is used for targets such as
     * VPC endpoints.
     * @default none
     */
    readonly endpoint?: IVpcEndpoint;
    constructor(props: RouteTargetProps);
}
/**
 * Interface to define a route.
 */
export interface IRouteV2 {
    /**
     * The ID of the route table for the route.
     * @attribute routeTable
     */
    readonly routeTable: IRouteTable;
    /**
     * The IPv4 or IPv6 CIDR block used for the destination match.
     *
     * Routing decisions are based on the most specific match.
     * TODO: Look for strong IP type implementation here.
     */
    readonly destination: string;
    /**
     * The gateway or endpoint targeted by the route.
     */
    readonly target: RouteTargetType;
}
/**
 * Properties to define a route.
 */
export interface RouteProps {
    /**
     * The ID of the route table for the route.
     * @attribute routeTable
     */
    readonly routeTable: IRouteTable;
    /**
     * The IPv4 or IPv6 CIDR block used for the destination match.
     *
     * Routing decisions are based on the most specific match.
     */
    readonly destination: string;
    /**
     * The gateway or endpoint targeted by the route.
     */
    readonly target: RouteTargetType;
    /**
     * The resource name of the route.
     * @default none
     */
    readonly routeName?: string;
}
/**
 * Creates a new route with added functionality.
 * @resource AWS::EC2::Route
 */
export declare class Route extends Resource implements IRouteV2 {
    /**
     * The IPv4 or IPv6 CIDR block used for the destination match.
     *
     * Routing decisions are based on the most specific match.
     */
    readonly destination: string;
    /**
     * The gateway or endpoint targeted by the route.
     */
    readonly target: RouteTargetType;
    /**
     * The route table for the route.
     * @attribute routeTable
     */
    readonly routeTable: IRouteTable;
    /**
     * The type of router the route is targetting
     */
    readonly targetRouterType: RouterType;
    /**
     * The route CFN resource.
     */
    readonly resource?: CfnRoute;
    constructor(scope: Construct, id: string, props: RouteProps);
}
/**
 * Properties to define a route table.
 */
export interface RouteTableProps {
    /**
     * The ID of the VPC.
     */
    readonly vpc: IVpcV2;
    /**
     * The resource name of the route table.
     * @default none
     */
    readonly routeTableName?: string;
}
/**
 * Creates a route table for the specified VPC
 * @resource AWS::EC2::RouteTable
 */
export declare class RouteTable extends Resource implements IRouteTable, IDependable {
    /**
     * The ID of the route table.
     */
    readonly routeTableId: string;
    /**
     * The route table CFN resource.
     */
    readonly resource: CfnRouteTable;
    constructor(scope: Construct, id: string, props: RouteTableProps);
}
