import { Resource } from 'aws-cdk-lib';
import { IVpc, ISubnet, SubnetSelection, SelectedSubnets, EnableVpnGatewayOptions, VpnConnectionOptions, VpnConnection, ClientVpnEndpointOptions, ClientVpnEndpoint, InterfaceVpcEndpointOptions, InterfaceVpcEndpoint, GatewayVpcEndpointOptions, GatewayVpcEndpoint, FlowLogOptions, FlowLog, CfnVPCCidrBlock } from 'aws-cdk-lib/aws-ec2';
import { IDependable } from 'constructs';
/**
 * Placeholder to see what extra props we might need,
 * will be added to original IVPC
 */
export interface IVpcV2 extends IVpc {
    /**
     * The secondary CIDR blocks associated with the VPC.
     *
     * For more information, see the {@link https://docs.aws.amazon.com/vpc/latest/userguide/vpc-cidr-blocks.html#vpc-resize}.
     */
    readonly secondaryCidrBlock: CfnVPCCidrBlock[];
    /**
     * The primary IPv4 CIDR block associated with the VPC.
     * Needed in order to validate the vpc range of subnet
     * current prop vpcCidrBlock refers to the token value
     * For more information, see the {@link https://docs.aws.amazon.com/vpc/latest/userguide/vpc-cidr-blocks.html#vpc-sizing-ipv4}.
     */
    readonly ipv4CidrBlock: string;
}
/**
 * Base class for creating a VPC (Virtual Private Cloud) in AWS.
 *
 * For more information, see the {@link https://docs.aws.amazon.com/cdk/api/v2/docs/aws-cdk-lib.aws_ec2.Vpc.html|AWS CDK Documentation on VPCs}.
 */
export declare abstract class VpcV2Base extends Resource implements IVpcV2 {
    /**
    * Identifier for this VPC
    */
    abstract readonly vpcId: string;
    /**
    * Arn of this VPC
    */
    abstract readonly vpcArn: string;
    /**
    * CIDR range for this VPC
    */
    abstract readonly vpcCidrBlock: string;
    /**
    * List of public subnets in this VPC
    */
    readonly publicSubnets: ISubnet[];
    /**
    * List of private subnets in this VPC
    */
    readonly privateSubnets: ISubnet[];
    /**
    * List of isolated subnets in this VPC
    */
    abstract readonly isolatedSubnets: ISubnet[];
    /**
    * AZs for this VPC
    */
    readonly availabilityZones: string[];
    /**
    * Dependable that can be depended upon to force internet connectivity established on the VPC
    */
    abstract readonly internetConnectivityEstablished: IDependable;
    /**
     * Secondary IPs for the VPC, can be multiple Ipv4 or Ipv6
     * Ipv4 should be within RFC#1918 range
     */
    abstract readonly secondaryCidrBlock: CfnVPCCidrBlock[];
    /**
     * The primary IPv4 CIDR block associated with the VPC.
     * Needed in order to validate the vpc range of subnet
     * current prop vpcCidrBlock refers to the token value
     * For more information, see the {@link https://docs.aws.amazon.com/vpc/latest/userguide/vpc-cidr-blocks.html#vpc-sizing-ipv4}.
     */
    abstract readonly ipv4CidrBlock: string;
    /**
    * If this is set to true, don't error out on trying to select subnets
    */
    protected incompleteSubnetDefinition: boolean;
    /**
     * Mutable private field for the vpnGatewayId
     *
     * @internal
     */
    protected _vpnGatewayId?: string;
    /**
    * Return information on the subnets appropriate for the given selection strategy
    *
    * Requires that at least one subnet is matched, throws a descriptive
    * error message otherwise.
    */
    selectSubnets(selection?: SubnetSelection): SelectedSubnets;
    /**
    * Adds a VPN Gateway to this VPC
    */
    enableVpnGateway(options: EnableVpnGatewayOptions): void;
    /**
    * Adds a new VPN connection to this VPC
    */
    addVpnConnection(id: string, options: VpnConnectionOptions): VpnConnection;
    /**
    * Adds a new client VPN endpoint to this VPC
    */
    addClientVpnEndpoint(id: string, options: ClientVpnEndpointOptions): ClientVpnEndpoint;
    /**
    * Adds a new interface endpoint to this VPC
    */
    addInterfaceEndpoint(id: string, options: InterfaceVpcEndpointOptions): InterfaceVpcEndpoint;
    /**
    * Adds a new gateway endpoint to this VPC
    */
    addGatewayEndpoint(id: string, options: GatewayVpcEndpointOptions): GatewayVpcEndpoint;
    /**
    * Adds a new flow log to this VPC
    */
    addFlowLog(id: string, options?: FlowLogOptions): FlowLog;
    /**
    * Returns the id of the VPN Gateway (if enabled)
    */
    get vpnGatewayId(): string | undefined;
    /**
    * Return the subnets appropriate for the placement strategy
    */
    protected selectSubnetObjects(selection?: SubnetSelection): ISubnet[];
    private applySubnetFilters;
    private selectSubnetObjectsByName;
    private selectSubnetObjectsByType;
    /**
    * Validate the fields in a SubnetSelection object, and reify defaults if necessary
    *
    * In case of default selection, select the first type of PRIVATE, ISOLATED,
    * PUBLIC (in that order) that has any subnets.
    */
    private reifySelectionDefaults;
}
